
#ifndef _SCRIPT_H
#define _SCRIPT_H

#include <functional>
#include <map>
#include <list>
#include <cstring>

#include "lg/objstd.h"
#include "lg/script.h"

#undef DispatchMessage

#define DECLARE_MODULE(osmName)
#define DECLARE_SCRIPT(clName,clBase)  class clName : public clBase
#define DECLARE_FACTORY(scrName,scrBase)  static IScript* Make##scrName (char* pszName, int iHostObjId)

#define DECLARE_MSGHANDLER(fnName)  long __stdcall fnName(sScrMsg*, sMultiParm*, eScrTraceAction)
#define REGISTER_MSGHANDLER(msgName,fnHandler)  \
	RegisterMessageHandler(msgName, static_cast<MessageHandlerFunc>(&fnHandler))


class cScript : public IScript
{
protected:
	typedef long __stdcall (cScript::*MessageHandlerFunc)(sScrMsg*, sMultiParm*, eScrTraceAction);

private:
	struct strless_nocase : public std::binary_function<const char*,const char*,bool>
	{
		bool operator()(const char* __x, const char* __y) const
			{ return ::stricmp(__x,__y) < 0; }
	};

	typedef std::map<const char*, std::list<MessageHandlerFunc>, strless_nocase>  tDispatchMap;

public:
	// IUnknown
	STDMETHOD(QueryInterface)(REFIID,void**);
	STDMETHOD_(ULONG,AddRef)(void);
	STDMETHOD_(ULONG,Release)(void);
	// IScript
	STDMETHOD_(const char*,GetClassName)(void);
	STDMETHOD_(long,ReceiveMessage)(sScrMsg*,sMultiParm*,eScrTraceAction);

private:
	int m_iRef;

public:
	virtual ~cScript();
	cScript();
	cScript(const char* pszName, int iHostObjId);

	STDMETHOD_(void*,GetThisPointer)()
	{
		return reinterpret_cast<void*>(this);
	}
	void DebugString(const char* pszMsg1, const char* pszMsg2 = "");

protected:
	char* m_szName;
	int m_iObjId;
	bool m_bSim;

	tDispatchMap m_mapHandlers;

	bool RegisterMessageHandler(const char* pszMsgName, MessageHandlerFunc pmfMsgHandler);
	bool UnregisterMessageHandler(const char* pszMsgName, MessageHandlerFunc pmfMsgHandler);
	long DispatchMessage(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace);
	virtual void InitScriptData()
	{
		RegisterMessageHandler("ScriptPtrQuery", static_cast<MessageHandlerFunc>(&cScript::ScriptPtrQuery));
	}

private:

	long __stdcall ScriptPtrQuery(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace);

};

#endif // _SCRIPT_H
