#!/usr/bin/perl -w

use strict;

my $OSMName;
my @Scripts;
my %History;

sub process_file
{
	my ($file,$fh,$ifile,$script);
	$file = shift or return;
	return if ($History{$file});
	$History{$file} = 1;
	open($fh,'<',$file) or die "blegh";
	my $skip = 0;
	while (<$fh>) {
		my ($osm,$class,$name,$base);
		if ($skip) {
			if (s|^.*\*/||) {
				$skip = 0;
			}
			else {
				next;
			}
			# This is way too simplistic. Do write simple comments.
		}
		s|//.*$||;
		s|/\*.*\*/||;
		if (s|/\*.*$||) {
			$skip = 1;
			next;
		}
		if (($osm) = m|^\s*DECLARE_MODULE\(\s*(\w+)\s*\)|) {
			$OSMName = $osm;
		}
		elsif (($class,$base) = m|^\s*DECLARE_SCRIPT\(\s*(\w+)\s*,\s*(\w+)\s*\)|) {
			$script = { 'ClassName' => $class };
			$script->{'ClassBase'} = $base;
		}
		elsif (($name,undef,$base) = m|^\s*DECLARE_FACTORY\(\s*(\w+)\s*,\s*((\w+)\s*)?\)|) {
			if ($script) {
				$script->{'ScriptName'} = $name;
				$script->{'BaseName'} = $base || "";
				push(@Scripts,$script);
				undef $script;
			}
		}
		elsif (($ifile) = m|^#include \"([-\.\w]+)\"|) {
			&process_file($ifile);
		}
	}
	close($fh);

	#@Scripts = sort {$a->{'ScriptName'} cmp $b->{'ScriptName'}} @Scripts;
	
}

sub output_header {
	my ($file,$ofile,$fh,$ofh);
	$ofile = shift or return;
	$file = shift;
	if ($ofile ne '--') {
		open($fh,">",$ofile) or die "damn";
		binmode($fh);
		$ofh = select $fh;
	}

	print <<L_TOP;
/*** Generated Automatically by genosm.pl -- modifications will be overwritten ***/
#include "ScriptModule.h"
#include "$file"
const sScrClassDesc cScriptModule::sm_ScriptsArray[] = {
L_TOP

	foreach my $s (@Scripts) {
		print "\t{ \"",$OSMName,"\", ",
			  "\"",$s->{'ScriptName'},"\", ",
			  "\"",$s->{'BaseName'},"\", ",
			  $s->{'ClassName'},"::Make",$s->{'ScriptName'},
			  " },\n";
	}
	print "};\nconst unsigned int cScriptModule::sm_ScriptsArraySize = ",$#Scripts + 1,";\n\n";

	foreach my $s (@Scripts) {
		print <<L_FCN;

IScript* __stdcall $s->{'ClassName'}::Make$s->{'ScriptName'}(char* pszName, int iHostObjId)
{
	$s->{'ClassName'}* pscrRet = new(std::nothrow) $s->{'ClassName'}(pszName, iHostObjId);
	if (pscrRet)
	{
		pscrRet->AddRef();
		pscrRet->InitScriptData();
	}
	return static_cast<IScript*>(pscrRet);
}
L_FCN
	}
	if ($ofile ne '--') {
		select $ofh;
		close($fh);
	}
}

{
	my ($fin,$fout);
	my $stdout = 0;
	while ($_ = $ARGV[0], /^-/) {
		shift;
		last if /^--$/;
		if (/^-c/) { $stdout = 1; }
		if (/^-o/) { $fout = shift; }
	}
	foreach my $f (@ARGV) {
		&process_file($f);
		$fin = $f;
	}
	if ($stdout) { $fout = '--'; }
	if (!defined $fout) { ($fout = $fin) =~ s|\.h$|_osm.cc|; }
	&output_header($fout,$fin);
}
